#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Utilities

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 13:53
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from .bot.text import TextBot
from .bot.markdown import MarkdownBot
from .bot.image import ImageBot
from .bot.news import NewsBot
from .bot.file import FileBot
from .bot.voice import VoiceBot
from .bot.template_card.text import TextCardBot
from .bot.template_card.news import NewsCardBot

__all__ = [
    "TextBot",
    "MarkdownBot",
    "ImageBot",
    "NewsBot",
    "FileBot",
    "VoiceBot",
    "TextCardBot",
    "NewsCardBot",
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Abstract classes

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 13:54
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
__all__ = ["AbstractBot", "FilePathLike"]

from abc import ABC, abstractmethod
from logging import getLogger
from os import PathLike
from pathlib import Path
from typing import Tuple, Dict, Union, List
from urllib.parse import quote, urlparse, parse_qs, urljoin
from uuid import UUID
from functools import partial

from requests import Session, session

from ..deco import verify_and_convert_arguments, verify_file
from ..deco import detect_overheat, handle_request_exception

logger = getLogger(__name__)
FilePathLike = Union[str, PathLike]
ConvertedData = Tuple[Tuple[Dict[str, str]], Dict[str, str]]


class _Basic(ABC):
    """Private bot basic class"""

    # The base path of the document
    _DOC_URL: str = "https://developer.work.weixin.qq.com/document/path/99110"
    # API Endpoint base url
    _API_BASE_URL: str = 'https://qyapi.weixin.qq.com'

    def __init__(self, key: str) -> None:
        """
        Initialize the class.
        :param key: The key of group bot webhook url.
        """
        if key is None:
            raise ValueError("Key is required")
        self.key = self._verify_uuid(key.strip())
        self._session: Session = session()

    @staticmethod
    def _parse_webhook_url(url: str) -> str:
        """
        Parse webhook url into key string.
        :param url: Webhook url.
        :return: Key string.
        """
        # If the key passed by url, split and parse it.
        try:
            query = urlparse(url).query
            params = parse_qs(query)
            if "key" not in params or not params["key"]:
                raise ValueError("Missing 'key' parameter in URL")
            return params["key"][0]
        except Exception as error:
            msg = f"Invalid webhook URL {url}."
            logger.critical(msg)
            raise ValueError(msg) from error

    def _verify_uuid(self, key: Union[str, UUID], max_attempts: int = 2) -> str:
        """
        Verify the key weather is UUID format.
        :param key: Key string
        :param max_attempts: Max number of attempts.
        :return: Result bool
        """
        if max_attempts <= 0:  # pragma: no cover
            raise ValueError("Maximum verification attempts exceeded")
        # The standard key format is UUID format.
        if isinstance(key, UUID):  # pragma: no cover
            return str(key)
        try:
            UUID(key)
            return key
        except (ValueError, TypeError, AttributeError) as error:
            try:
                key = self._parse_webhook_url(key)
                return self._verify_uuid(key, max_attempts - 1)
            except ValueError:
                ...
            raise ValueError(f"Invalid key format: {key}") from error

    def __repr__(self) -> str:
        """
        Return the class name.
        :return: Class name.
        """
        return f"{self.__class__.__name__}({self.key})"

    @property
    def _api_url(self) -> str:
        """
        Returns the address of the spliced endpoint url.
        :return: Endpoint url
        """
        end_point = urljoin(self._API_BASE_URL, self._api_path)
        return end_point

    @property
    @abstractmethod
    def _api_path(self) -> str:
        """
        The path of the API Endpoint.
        :return: path of the API Endpoint.
        """

    @property
    @abstractmethod
    def _doc_key(self) -> str:
        """
        The key of the document description.
        :return: key of the document description
        """

    @property
    def doc(self) -> str:
        """
        API URL of the document description
        :return: URL of the document
        """
        url = f"{self._DOC_URL}#{quote(self._doc_key)}"
        return url


class _MediaUploader(_Basic):
    """Upload voice and file class"""

    @property
    def _api_path(self) -> str:
        """
        The path of the API Endpoint.
        :return: path of the API Endpoint.
        """
        return "cgi-bin/webhook/upload_media"

    @property
    def _doc_key(self) -> str:
        """
        The key of the document description.
        :return: key of the document description
        """
        return "文件上传接口"

    @handle_request_exception
    def _upload_temporary_file(self, file_path: Path, file_type: str) -> dict:
        """
        Upload temporary file from file path.
        :param file_path: File path.
        :param file_type: File type.
        :return:
        """
        kwargs = {
            "url": self._api_url,
            "params": {
                "key": self.key,
                "type": file_type
            }
        }
        cmd = partial(self._session.post, **kwargs)
        with open(file_path, "rb") as files:
            response = cmd(files={"media": files})
        result = response.json()
        return result

    @verify_file
    def upload(self, file_path: FilePathLike, /, **kwargs) -> str:
        """
        Upload voice and file.
        :param file_path: The path of the file.
        :param kwargs: Other keyword arguments.
        :return: Result dict.
        """
        file_path: Path = Path(file_path)
        # Only support `AMR` format for voice, others must be file type
        file_type: str = "voice" if file_path.suffix == ".amr" else "file"
        logger.debug("~~~~ %s ~~~~", self.upload.__name__)
        logger.debug("File path: %s", file_path)
        logger.debug("Other kwargs: %s", kwargs)
        logger.debug("API endpoint URL: %s", self._api_url)
        result = self._upload_temporary_file(file_path, file_type)
        logger.debug("~~~~ %s ~~~~", self.upload.__name__)
        logger.info("%s has been uploaded: %s", file_type.capitalize(), result)
        return result["media_id"]


class AbstractBot(_Basic, ABC):
    """Abstract class of Wecom group bot."""

    # Default requests headers
    _HEADERS: dict = {"Content-Type": "application/json"}
    # Class level variable for detect overheat
    _OVERHEAT: int = -1

    def __init__(self, key: str) -> None:
        _Basic.__init__(self, key)
        ABC.__init__(self)
        self._session.headers = self._HEADERS
        self._uploader: _MediaUploader = _MediaUploader(key)

    @property
    def _api_path(self) -> str:
        """
        The path of the API Endpoint.
        :return: path of the API Endpoint.
        """
        return "cgi-bin/webhook/send"

    @property
    @abstractmethod
    def _doc_key(self) -> str:
        """
        The key of the document description.
        :return: key of the document description
        """

    # pylint:disable=unused-argument
    @handle_request_exception
    @detect_overheat
    @verify_and_convert_arguments
    def send(self,
             msg: str = None,
             /,
             articles: List[Dict[str, str]] = None,
             file_path: FilePathLike = None,
             **kwargs) -> dict:
        """
        Method of sending a message. `Refer`_

        .. _`Refer`: https://developer.work.weixin.qq.com/document/path/91770

        :param msg: Message body.
        :param articles: List of articles. Used for send news.
        :param file_path: File path. Used for send image/voice/file.
        :return: Result dict.
        """
        logger.debug("~~~~ %s ~~~~", self.send.__name__)
        logger.debug("Message: %s", msg)
        logger.debug("Articles: %s", articles)
        logger.debug("File path: %s", file_path)
        logger.debug("Other kwargs: %s", kwargs)
        logger.debug("API endpoint URL: %s", self._api_url)
        response = self._session.post(self._api_url,
                                      params={"key": self.key},
                                      json=msg)
        result = response.json()
        logger.debug("~~~~ %s ~~~~", self.send.__name__)
        logger.info("Message has been sent: %s", result)
        return result

    def upload(self, file_path: FilePathLike) -> str:
        """
        Upload temporary media file
        :param file_path: The path of the file to upload.
        :return:
        """
        result = self._uploader.upload(file_path)
        return result

    @abstractmethod
    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify arguments methods. Subclasses must complete specific implementations
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return: None
        """

    @abstractmethod
    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Prepare data methods, subclasses must complete specific implementations
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Result dict.
        """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Text Card type message sender

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/6/4 10:09
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from functools import partial
from typing import List

from jmespath import search

from . import TemplateCardKeys, TemplateCardRequirements
from .. import AbstractBot, ConvertedData


class TextCardBot(AbstractBot):
    """Text Card type message Wecom Group Bot"""

    _VALID_KEYS: List[str] = TemplateCardKeys + [
        "emphasis_content", "sub_title_text"
    ]

    @property
    def _doc_key(self) -> str:
        return "文本通知模版卡片"

    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return:
        """
        reqs = {
            "Either `main_title.title` should be existed or `sub_title_text` be existed":
                partial(
                    search, """
                    (main_title.title == null || main_title.title == '') &&
                    (sub_title_text == null || sub_title_text == '')
                """),
            **TemplateCardRequirements
        }
        for msg, cmd in reqs.items():
            if cmd(kwargs):
                raise ValueError(msg)

    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to text card format data.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted data.
        """
        kw_ = {
            key: val for key, val in kwargs.items() if key in self._VALID_KEYS
        }
        result = ({
            "msgtype": "template_card",
            "template_card": {
                "card_type": "text_notice",
                **kw_,
            }
        },)
        return result, kwargs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Wecom(A.K.A. WeChat Work) Group Bot python API.


- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 13:42
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

__author__ = 'Rex Zhou'
__copyright__ = 'Copyright © 2025 Rex Zhou. All rights reserved.'
__credits__ = [__author__]
__license__ = "MIT"
__maintainer__ = __author__
__email__ = '879582094@qq.com'

from .utils import TextBot
from .utils import MarkdownBot
from .utils import ImageBot
from .utils import NewsBot
from .utils import FileBot
from .utils import VoiceBot
from .utils import TextCardBot
from .utils import NewsCardBot



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Markdown type message sender


- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 15:12
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from enum import Enum
from typing import Union, Callable
from functools import partial

from . import AbstractBot, ConvertedData


class MarkdownBot(AbstractBot):
    """Markdown type message Wecom Group Bot"""

    class _Color(Enum):
        """Markdown _color enum"""
        INFO = "green"
        COMMENT = "gray"
        WARNING = "orange"

        @classmethod
        def get_valid_colors(cls):
            """Return list of valid colors"""
            return [_.value for _ in cls]

    @property
    def _doc_key(self) -> str:
        return "markdown类型"

    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments passed.
        :param kwargs: Keyword arguments passed.
        :return:
        """
        try:
            args[0]
        except IndexError as error:
            raise ValueError("The msg parameter is required.") from error

    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to Markdown format data.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted data.
        """
        result = ({
            "msgtype": "markdown",
            "markdown": {
                "content": args[0].strip()
            }
        },)
        return result, kwargs

    def _color(self, raw: str, color: Union[str, _Color]) -> str:
        """
        Convert normal string to colorful string.
        :param raw: Raw string.
        :param color: Specify _color. Support: green | gray | orange
        :return: Colorized string.
        """
        if isinstance(color, str):
            try:
                color = self._Color(color.lower())
            except ValueError as error:
                valid_colors = self._Color.get_valid_colors()
                raise ValueError(
                    f"Invalid color '{color}'. Valid options: {valid_colors}"
                ) from error
        result = f'<font color="{color.name.lower()}">{raw}</font>'
        return result

    @property
    def green(self) -> Callable:
        """
        Return a function that green text
        :return: 
        """
        return partial(self._color, color=self._Color.INFO)

    @property
    def gray(self) -> Callable:
        """
        Return a function that gray text
        :return:
        """
        return partial(self._color, color=self._Color.COMMENT)

    @property
    def orange(self) -> Callable:
        """
        Return a function that orange text
        :return:
        """
        return partial(self._color, color=self._Color.WARNING)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Test error module

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 14:58
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

from logging import DEBUG, basicConfig
from os import getenv
from pathlib import Path

from dotenv import load_dotenv
from pytest import raises
from yaml import safe_load

# pylint: disable=import-error
from src.pywgb import FileBot
from src.pywgb import ImageBot
from src.pywgb import MarkdownBot
from src.pywgb import NewsBot
from src.pywgb import TextBot
from src.pywgb import VoiceBot
from src.pywgb import TextCardBot
from src.pywgb import NewsCardBot
from tests.test_main import env_file, errors_file, TEST_VALID_ARTICLES

basicConfig(level=DEBUG, format="%(levelname)s %(name)s %(lineno)d %(message)s")
load_dotenv(env_file, override=True)


def test_overheat() -> None:
    """
    Test overheat function
    :return:
    """
    bot = TextBot(getenv("VALID_KEY"))
    bot.send("This message was delayed by overheat", test="overheat")


def test_request_exception() -> None:
    """
    Test request exception
    :return:
    """
    bot = TextBot(getenv("VALID_KEY"))
    with raises(ConnectionRefusedError) as exception_info:
        bot.send("This message WON'T be sent, cause by request error",
                 test="request_error")
    assert "Unable to initiate API request correctly" in str(
        exception_info.value)
    with raises(IOError) as exception_info:
        bot.send("This message WON'T be sent, cause by API error",
                 test="api_error")
    assert "Request failed, please refer to the official manual" in str(
        exception_info.value)


def test_verify_text_error() -> None:
    """
    Test TEXT verification error.
    :return:
    """
    bot = TextBot(getenv("VALID_KEY"))
    with raises(ValueError) as error:
        bot.send()
    assert "The msg parameter is required" in str(error.value)
    err_msg = "parameter should be a list of strings"
    tests = [
        [123],
        (123,),
    ]
    for test in tests:
        with raises(ValueError) as error:
            bot.send("This message won't be sent, cause verify error.",
                     mentioned_mobile_list=test)
        assert err_msg in str(error.value)


def test_verify_markdown_error() -> None:
    """
    Test MARKDOWN verification error.
    :return:
    """
    bot = MarkdownBot(getenv("VALID_KEY"))
    with raises(ValueError) as error:
        bot.send()
    assert "The msg parameter is required" in str(error.value)
    with raises(ValueError) as error:
        # pylint: disable=protected-access
        bot._color("This will raise an exception", "red")
    assert "Invalid _color" in str(error.value)


def test_verify_news_error() -> None:
    """
    Test News verification error.
    :return:
    """
    bot = NewsBot(getenv("VALID_KEY"))
    # Test empty articles
    with raises(ValueError) as exception_info:
        bot.send()
    assert "The articles parameter is required" in str(exception_info.value)
    with raises(ValueError) as exception_info:
        bot.send(articles=[])
    assert "The articles parameter is empty" in str(exception_info.value)
    # Test oversize articles
    with raises(ValueError) as exception_info:
        articles = [TEST_VALID_ARTICLES[0] for _ in range(9)]
        bot.send(articles=articles)
    assert "Too many articles." in str(exception_info.value)
    # Test data error and parameter error
    tests = {
        "article_data_error": "data is not a dict",
        "article_parameter_error": "lack required parameter",
    }
    for code, msg in tests.items():
        with raises(ValueError) as exception_info:
            bot.send(articles=TEST_VALID_ARTICLES, test=code)
        assert msg in str(exception_info.value)


def test_verify_image_error() -> None:
    """
    Test image verification error.
    :return:
    """
    bot = ImageBot(getenv("VALID_KEY"))
    with raises(ValueError) as exception_info:
        bot.send()
    assert "The file_path parameter is required" in str(exception_info.value)
    file = Path(__file__).with_name("test.png")
    mapper = {
        "wrong_format_image": "Just support image type: jpg or png",
        "oversize_image": "The image is too large, more than 2M",
    }
    for code, msg in mapper.items():
        with raises(ValueError) as exception_info:
            bot.send(file_path=file, test=code)
        assert msg in str(exception_info.value)


def test_verify_file_error() -> None:
    """
    Test verify file error.
    :return:
    """
    bot = FileBot(getenv("VALID_KEY"))
    # Test lack file
    with raises(ValueError) as exception_info:
        bot.send()
    assert "The file_path parameter is required" in str(exception_info.value)
    # Test empty file
    with raises(ValueError) as exception_info:
        bot.send(file_path=None)
    assert "The file_path parameter is required" in str(exception_info.value)
    # Test invalid file
    with raises(ValueError) as exception_info:
        bot.send(file_path="234234234.txt")
    assert "The file_path parameter not exists" in str(exception_info.value)
    # Test oversize file
    file = Path(__file__).with_name("test.png")
    with raises(ValueError) as exception_info:
        bot.send(file_path=file, test="oversize_file")
    assert "The file size is out of range" in str(exception_info.value)


def test_verify_voice_error() -> None:
    """
    Test voice verification error.
    :return:
    """
    bot = VoiceBot(getenv("VALID_KEY"))
    # Test lack file
    with raises(ValueError) as exception_info:
        bot.send()
    assert "The file_path parameter is required" in str(exception_info.value)
    # Test invalid file format
    file = Path(__file__).with_name("test.png")
    with raises(ValueError) as exception_info:
        bot.send(file_path=file, test="wrong_format_voice")
    assert "Just support voice type: amr" in str(exception_info.value)
    # Test overlong and oversize voice
    file = Path(__file__).with_name("test.amr")
    tests = {
        "oversize_voice": "The voice size is out of range",
        "overlong_voice": "The voice duration is longer than 60s",
    }
    for code, msg in tests.items():
        with raises(ValueError) as exception_info:
            bot.send(file_path=file, test=code)
        assert msg in str(exception_info.value)


def test_verify_text_card_error() -> None:
    """
    Test Text Card verification error.
    :return:
    """
    bot = TextCardBot(getenv("VALID_KEY"))
    with open(errors_file, "r", encoding="utf-8") as _:
        tests = safe_load(_)
    for err_msg, kwargs in tests["text"].items():
        with raises(ValueError) as error:
            bot.send(**kwargs)
        assert err_msg in str(error.value)


def test_verify_news_card_error() -> None:
    """
    Test News Card verification error.
    :return:
    """
    bot = NewsCardBot(getenv("VALID_KEY"))
    with open(errors_file, "r", encoding="utf-8") as _:
        tests = safe_load(_)
    for err_msg, kwargs in tests["news"].items():
        with raises(ValueError) as error:
            bot.send(**kwargs)
        assert err_msg in str(error.value)
